// HTTP Packet Listener

// Copyright (C) 1999 by Makoto Nagata, MicroNova <mnagata@micronova.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit http://www.micronova.com for updates

package com.micronova.util;

import com.micronova.util.Packet;

import netscape.application.*;
import netscape.util.*;
import java.net.*;
import java.io.*;

/**

   HTTPPacketListener is a thread Runnable for listening to an HTTP
   connection for incoming stream of Codables.  When run, it first
   makes a connection to the given url, and sends the given parameter.
   Then whenever a Codable is received or an exception occurs, it
   asynchronously posts a message addressed to the given target
   in the application's queue.
   <BR>
   <BR>
   The target's performCommand(command, object) is called as follows:
   <UL>
   <LI>
   for a received data, command = HTTPPacketListener.DATA, object = the received Codable;
   <LI>
   for an exception, command = HTTPPacketListener.EXCEPTION, object = the Exception object.
   </UL>

   * @version 1.0
   * @author Makoto Nagata, MicroNova

*/

public class HTTPPacketListener implements Runnable
{
    /** Application that provides a message queue. */

    protected Application application;

    /** Target that receives messages. */

    protected Target target;

    /** URL to listen to. */

    protected URL url;

    /** Codable parameter to be sent to the URL before listening starts. */

    protected Codable parameter;

    /** set to false to abort listening. */

    protected boolean isRunning = true;

    /** command sent when a packet is received. */

    public static final String DATA = "Data.Packet.util.micronova.com";

    /** command sent when an exception occurs. */

    public static final String EXCEPTION = "Exception.Packet.util.micronova.com";
    /**

       <BR>Constructs an HTTPPacketListener.
       <BR>
       <BR>
       * @param url a URL to listen to
       * @param parameter a Codable that is sent to the URL before listening starts (can be null)
       * @param application an Application that supplies a message queue
       * @param target a Target that receives messages
       
    */

    public HTTPPacketListener(URL url, Codable parameter, Application application
, Target target)
    {
        this.application = application;
        this.target = target;
        this.url = url;
        this.parameter = parameter;
    }

    public void run()
    {
        InputStream in = null;
        OutputStream out = null;

        try
        {
            URLConnection connection = url.openConnection();
    
            connection.setUseCaches(false);

            connection.setDoInput(true);
            connection.setDoOutput(true);

            // send argument

            out = connection.getOutputStream();

            Packet.write(out, parameter);
    
            out.close();

            out = null;
    
            in = connection.getInputStream();

            while (isRunning)
            {
                Codable codable = Packet.read(in);

                if (codable != null)
                {
                    application.performCommandLater(target, DATA, codable);
                }
            }
        }
        catch (Exception e)
        {
            application.performCommandLater(target, EXCEPTION, e);
            
            try
            {
                if (out != null)
                {
                    out.close();
                }
                
                if (in != null)
                {
                    in.close();
                }
            }
            catch (Exception e1)
            {
            }
        }
    }

    /**

       Aborts listening.  The thread will exit.

    */

    public void abort()
    {
        isRunning = false;
    }
}

